
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <math.h>

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#include <GL/gl.h>

#include "c3ga_util.h"

/// Multiplies the current OpenGL matrix with versor 'v'
void unit_versor_glMult(const c3ga &v) {
	// compute the images of all OpenGL basis vectors
	c3ga image[4] = {
			(v * (c3ga::e1 ^ c3ga::ni) * v.inverse())(GRADE2), // image of e1
			(v * (c3ga::e2 ^ c3ga::ni) * v.inverse())(GRADE2), // image of e2
			(v * (c3ga::e3 ^ c3ga::ni) * v.inverse())(GRADE2), // image of e3
			(v * (c3ga::no ^ c3ga::ni) * v.inverse())(GRADE2)  // image of origin
	};
/*
	printf("\nversor: %s\n", v.string());
	printf("versor * no / versor: %s\n", (v * c3ga::no / v).string());
	printf("image e1: %s\n",  image[0].string());
	printf("image e2: %s\n",  image[1].string());
	printf("image e3: %s\n",  image[2].string());
	printf("image o: %s\n",  image[3].string());
*/

	// transfer the coordinates to the OpenGL matrix
	GLfloat GLmatrix[16];
	for (int i = 0; i < 4; i++) {
		const GAIM_FLOAT *c = image[i][GRADE2];
		GLmatrix[i * 4 + 0] = (GLfloat)c[C3GA_E1_NI];
		GLmatrix[i * 4 + 1] = (GLfloat)c[C3GA_E2_NI];
		GLmatrix[i * 4 + 2] = (GLfloat)c[C3GA_E3_NI];
		GLmatrix[i * 4 + 3] = (GLfloat)c[C3GA_NO_NI];
	}

	/*printf("The matrix has you: \n");
	for (int i = 0; i < 4; i++) {
		for (int j = 0; j < 4; j++) {
			printf("%f ", GLmatrix[j * 4 + i]);
		}
		printf("\n");
	}*/

	// multiply current OpenGL matrix:
	glMultMatrixf(GLmatrix);
}

void dissect_point_pair(const c3ga &pp, c3ga &pt1, c3ga &pt2) {
	GAIM_FLOAT n = sqrt((pp << pp).scalar());
	c3ga v1(c3ga::ni << pp);
	c3ga v2(n * v1);
	c3ga v3(v1 << pp);

	pt1 = v3 + v2;
	pt2 = v3 - v2;
}

void dissect_point_pair_normalized_flat(const c3ga &pp, c3ga &fpt1, c3ga &fpt2) {
	GAIM_FLOAT n = sqrt((pp << pp).scalar());
	c3ga v1(c3ga::ni << pp);
	c3ga v2(n * v1);
	c3ga v3(v1 << pp);

	GAIM_FLOAT scale = 1.0 / ((v2 << c3ga::ni).scalar() + (v3 << c3ga::ni).scalar());
	fpt1 = (scale * c3ga::ni) ^ (v3 + v2);
	fpt2 = (scale * c3ga::ni) ^ (v3 - v2);
}

/*
void dissect_point_pair_flat(const c3ga &pp, c3ga &fp1, c3ga &fp2) {
// NOT FINISHED
	c3ga c = ((c3ga::ni << pp) << pp) ^ c3ga::ni; // c != unit

	c3ga A = c3ga::ni % (pp ^ c3ga::ni) / ((c3ga::no ^ c3ga::ni) % pp).scalar();          // A = attitude of point pair
        
	fp1 = (c / ((c3ga::no ^ c3ga::ni) % c).scalar()) + 0.5 * A;
    fp2 = (c / ((c3ga::no ^ c3ga::ni) % c).scalar()) - 0.5 * A;
}*/

