
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef _VERTEX_H_
#define _VERTEX_H_

#include "classes.h"

class vertex {
public:
	vertex(); ///< null constructor
	vertex(const c3ga &pt, const c3ga &att, const c3ga &tex_pt); ///< constructor; sets all data of the vertex
	vertex(const vertex &v); ///< copy constructor
	~vertex(); ///< destructor

	vertex &operator=(const vertex &v); ///< copy operator

	/// retrieves m_pt
	inline const c3ga &get_pt() const {return m_pt;} 
	/// retrieves m_att
	inline const c3ga &get_att() const {return m_att;}
	/// retrieves m_tex_pt
	inline const c3ga &get_tex_pt() const {return m_tex_pt;}

	/// calls glVertex() for OpenGL rendering
	void glVertex() const;
	/// calls glNormal() for OpenGL rendering
	void glNormal() const;

private:
	c3ga m_pt; ///< point in conformal model (CGA point)
	c3ga m_att; ///< local surface attitude (free bivector)
	c3ga m_tex_pt; ///< index into texture ((flat?) 2D point)
};

#endif /* _VERTEX_H_ */

