
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "vertex.h"
#include "c3ga_util.h"

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#include <GL/gl.h>

using namespace c3ga;

vertex::vertex() {
}

vertex::vertex(const c3ga::tangentBivector &pt_att, const normalizedFlatPoint2D &tex_pt) :
	m_pt_att(pt_att), m_tex_pt(tex_pt) {
}

vertex::vertex(const vertex &v) :
	m_pt_att(v.m_pt_att), m_tex_pt(v.m_tex_pt) {
}

vertex::~vertex() {
}

vertex &vertex::operator=(const vertex &v) {
	if (this != &v) {
		m_pt_att = v.m_pt_att;
		m_tex_pt = v.m_tex_pt;
	}
	return *this;
}

/// calls glVertex() for OpenGL rendering
void vertex::glVertex() const {
	// convert regular point in flat point representation:
	normalizedFlatPoint fpt(_normalizedFlatPoint(get_pt(get_pt_att()) ^ ni));

	// call OpenGL to specify vertex coordinates:
	glVertex4f(_float(fpt & e1ni),
		_float(fpt & e2ni),
		_float(fpt & e3ni),
		_float(fpt & noni));
}

/// calls glNormal() for OpenGL rendering
void vertex::glNormal() const {
	// get the local surface normal (dual of the attitude)
	freeVector normal = _freeVector(dual(get_att(get_pt_att())));

	// call OpenGL to specify normal coordinates:
	glNormal3f(_float(normal & e1ni),
		_float(normal & e2ni),
		_float(normal & e3ni));
}
