
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "camera.h"
#include "../shared.src/string_util.h"

using namespace c3ga;

camera::camera() : m_fov_width(0.0) {
	 m_xf.setVizData("cameraXF");
	 m_xfi.setVizData("cameraXFi");
	m_position.setVizData("cameraXFpt");
}	

camera::camera(const EXForm &xf,
		mv::Float fovWidth) : locator(xf), 
		m_fov_width(fovWidth) {
	 m_xf.setVizData("cameraXF");
	 m_xfi.setVizData("cameraXFi");
	m_position.setVizData("cameraXFpt");
}

camera::camera(const camera &c) : locator(c), 
	m_fov_width(c.m_fov_width) {
	 m_xf.setVizData("cameraXF");
	 m_xfi.setVizData("cameraXFi");
	m_position.setVizData("cameraXFpt");
}

camera::~camera() {

}

camera &camera::operator=(const camera& c) {
	if (this != &c) {
		locator::operator=(c);
		m_fov_width = c.m_fov_width;
	}
	return *this;
}

void camera::set_fov_width(mv::Float fov_width) {
	if (fov_width < 0.001) fov_width = 0.001f;
	m_fov_width = fov_width;
}

std::string camera::to_string() const {
	std::string result;

	result += "camera_begin\n";

	// xf
	result += "camera_xf " + locator::to_string() + "\n";

	// fov
	result += "camera_fov " + ::to_string(get_fov_width()) + "\n";

	result += "camera_end\n";

	return result;
}
