
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


/*
Command line options:
-r : just render
-s .... load scene '....'

*/
#include <stdio.h>
#include <math.h>
#include <string>

#include "ui.h"
#include "../shared.src/pngfe.h"

#include "../shared.src/mt19937-2.h"
#include "../shared.src/fltk_stuff.h"
#include "../shared.src/timing.h"
#include "version.h"

#include "c3ga_util.h"
using namespace c3ga;

FILE *g_vizOutputF = NULL;
void myVizCallback(const mv &value, const void *varData, const void *appData, bool assign) {
	if (g_vizOutputF == NULL)
		g_vizOutputF = fopen("e:/ray_c3ga_gaigen2_nice_viz.txt", "w+");
	const char *varName = "anonymous_variable";
	if (varData) varName = (const char*)varData;
	if (varData) {
		if ((varName[0] <= ' ') || (varName[0] >= '127'))
			printf("AHrg!\n");
		fprintf(g_vizOutputF, "_assign = %d; %s = %s,\n", assign, varName, c3ga::string(value, "%2.20e"));
//		fflush(g_vizOutputF);
	}
}


int main(int argc, char *argv[]) {


	int i;
	bool interactive = true;
	std::string sceneFilename;

	// parse simple command line options
	for (i = 1; i < argc; i++) {
		if (argv[i] == NULL) continue;
		else if (std::string(argv[i]) == "-r") {// just render
			argv[i] = "";
			interactive = false;
		}
		else if ((i < (argc-1)) && (std::string(argv[i]) == "-s")) {//  load scene [i+1]
			sceneFilename = argv[i+1];
			argv[i] = argv[i+1] = NULL;
		}
	}

	// remove used command line options:
	int j = 0;
	for (i = 0; i < argc; i++) {
		if (argv[i]) argv[j++] = argv[i];
	}
	argc = j;

	// check sanity of command line arguments
	if ((interactive == false) && (sceneFilename.size() == 0)) {
		printf("Need a scene filename to render. Use:\n"
			"%s -r -s filename.rsn\n", argv[0]);
		return -1;
	}

	scene S;

	// load scene, if any . . .
	if (sceneFilename.size()) {
		S.load_rsn(sceneFilename);
	}


	// todo: what is causing the second callback?
	/// temp:
/*	c3ga::point pppp = c3ga::_point(c3ga::e1);
	pppp.setVizData("pppp", true);
	fflush(g_vizOutputF);*/
	

	printf("Scene loaded\n");
	// temp
	const int w = 8, h = 8;
	std::vector<unsigned char>image(w * h * 3);
	S.render_init();
    u_timeElapsed();
	int verbosity = 1;
	c3ga::setVizCallback(myVizCallback, NULL);
	try {
		c3ga::g2Profiling::init();
	} catch (std::string &str){
		printf("Profile error: %s\n", str.c_str());
	}
//	vectorE3GA vec = _vectorE3GA(pointPair());
//	vectorE3GA vec2 = _vectorE3GA(pointPair());
	mv x = (no - e3) ^ e2;
//	mv y = (e1 - e3) ^ e2;

	c3ga::g2Profiling::reset();
	S.render(0, &(image[0]), w, h, 1, verbosity, NULL, 4);
	c3ga::g2Profiling::save();
	printf("%f secs\n", u_timeElapsed());
	// save result
	{
		std::string imageFilename;
		std::string extension = std::string("_") + g_ray_version + ".png";
		if (sceneFilename.substr(sceneFilename.size()-4) == ".rsn")
			imageFilename = sceneFilename.substr(0, sceneFilename.size()-4) + extension;
		else imageFilename = sceneFilename + extension;

		save_png(imageFilename.c_str(), &(image[0]), w, h);
	}

	if (g_vizOutputF) {
		fclose(g_vizOutputF);
		g_vizOutputF = NULL;
	}
		

	return 0;
	// end of temp
	

	if (interactive) {
		// create user interface . . .
		try {
			ui_state UI(S, argc, argv, "../files");
			UI.run();
		}
		catch (const std::string &S) {
			printf("Unexpected expection: %s\n", S.c_str());
		}
		catch (...) {
			printf("Unknown unexpected expection\n");
		}
	}
	else {
		// just render . . . 
		// S.render(...)
	}


	return 0;
}
