
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <math.h>

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#include <GL/gl.h>

#include "c3ga_util.h"
#include "e3ga.h"

void dissect_point_pair(const c3ga &pp, c3ga &pt1, c3ga &pt2) {
	GAIM_FLOAT n = sqrt((pp << pp).scalar());
	c3ga v1(c3ga::ni << pp);
	c3ga v2(n * v1);
	c3ga v3(v1 << pp);

	pt1 = v3 + v2;
	pt2 = v3 - v2;
}

void dissect_point_pair_normalized_flat(const c3ga &pp, c3ga &fpt1, c3ga &fpt2) {
	GAIM_FLOAT n = sqrt((pp << pp).scalar());
	c3ga v1(c3ga::ni << pp);
	c3ga v2(n * v1);
	c3ga v3(v1 << pp);

	GAIM_FLOAT scale = 1.0f / ((v2 << c3ga::ni).scalar() + (v3 << c3ga::ni).scalar());
	fpt1 = (scale * c3ga::ni) ^ (v3 + v2);
	fpt2 = (scale * c3ga::ni) ^ (v3 - v2);
}

void conformal_TSR_versor_to_3DGA(const c3ga &TSR, e3ga::vector &T, e3ga::mv::Float &S, e3ga::rotor &R) {
	// extract translation
	c3ga ifo= TSR * (c3ga::no ^ c3ga::ni) / TSR; // ifo =  image of flat origin
	T.set(e3ga::vector_e1_e2_e3, ifo[GRADE2][C3GA_E1_NI], ifo[GRADE2][C3GA_E2_NI], ifo[GRADE2][C3GA_E3_NI]);

	// extract scaling
	c3ga ii = TSR * (c3ga::ni) / TSR; // ii = image of infinity
	S = ii[GRADE1][C3GA_NI];
	
	// extract rotor
	c3ga VR = (-0.5 * log(S) * (c3ga::no ^ c3ga::ni)).exp() * (0.5 * (ifo - (c3ga::no^c3ga::ni))).exp() * TSR;
	R.set(e3ga::rotor_scalar_e1e2_e2e3_e3e1,
		VR.scalar(), VR[GRADE2][C3GA_E1_E2], VR[GRADE2][C3GA_E2_E3], -VR[GRADE2][C3GA_E1_E3]);


	// extract rotation matrix
/*	c3ga ie1 = TSR * c3ga::e1 / TSR; // ie1 = image of e1
	c3ga ie2 = TSR * c3ga::e2 / TSR; // ie2 = image of e2
	c3ga ie3 = TSR * c3ga::e3 / TSR; // ie3 = image of e3
	R.m11 = ie1[GRADE1][C3GA_E1]; R.m21 = ie1[GRADE1][C3GA_E2]; R.m31 = ie1[GRADE1][C3GA_E3];
	R.m12 = ie2[GRADE1][C3GA_E1]; R.m22 = ie2[GRADE1][C3GA_E2]; R.m32 = ie2[GRADE1][C3GA_E3];
	R.m13 = ie3[GRADE1][C3GA_E1]; R.m23 = ie3[GRADE1][C3GA_E2]; R.m33 = ie3[GRADE1][C3GA_E3];*/

}

	//V = exp(-0.5 (e1 - e2 + 0.5 e3) ^ ni) exp(0.5 log(0.77) no ^ ni) exp(-0.1 e1 ^ e2 + 0.2 e2 ^ e3 + 0.35 e3 ^ e1)
/*	c3ga V = 9.228068e-001 + -9.798043e-002*(c3ga::e1^c3ga::e2 ) + -3.429315e-001*(c3ga::e1^c3ga::e3 ) + 1.959609e-001*(c3ga::e2^c3ga::e3 ) + -5.628769e-001*(c3ga::e1^c3ga::ni ) + 6.320722e-001*(c3ga::e2^c3ga::ni ) + 4.377907e-002*(c3ga::e3^c3ga::ni ) + -1.199128e-001*(c3ga::no^c3ga::ni ) + 1.107124e-001*(c3ga::e1^c3ga::e2^c3ga::e3^c3ga::ni ) + 1.273192e-002*(c3ga::e1^c3ga::e2^c3ga::no^c3ga::ni ) + 4.456172e-002*(c3ga::e1^c3ga::e3^c3ga::no^c3ga::ni ) + -2.546384e-002*(c3ga::e2^c3ga::e3^c3ga::no^c3ga::ni);
	e3ga::vector T;
	e3ga::mv::Float S;
	e3ga::rotor R;
	conformal_TSR_versor_to_3DGA(V, T, S, R);
	printf("T: %s\n", e3ga::string(T, "%f"));
	printf("S: %f\n", S);
	printf("R: %s\n", e3ga::string(R, "%f"));*/


