
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef _LOCATOR_H_
#define _LOCATOR_H_

#include "classes.h"
#include "c3ga.h"

class locator {
public:
	locator();
	locator(const c3ga &TSR); ///< constructor from conformal TSR versor (uses conformal_TSR_versor_to_3DLA to convert)
	locator(const e3ga::vector &T, const GAIM_FLOAT S, const e3ga::rotor &R);
	locator(const locator &l);
	virtual ~locator();

	locator &operator=(const locator &l);

	void set(const e3ga::vector &T, const GAIM_FLOAT S, const e3ga::rotor &R); ///< sets transform
	void set_xf(const c3ga &TSR); ///< sets transform from conformal TSR versor (uses conformal_TSR_versor_to_3DLA to convert)

	inline const e3ga::vector &get_translation() const {
		return m_t;
	}

	inline const e3ga::rotor &get_rotation() const {
		return m_r;
	}

	inline e3ga::mv::Float get_scale() const {
		return m_s;
	}

	inline e3ga::vector transform_point(const e3ga::vector &point) const {
		return e3ga::_vector(m_s * (m_r * point * m_ri) + m_t);
	}

	inline e3ga::vector inverse_transform_point(const e3ga::vector &point) const {
		return e3ga::_vector(m_ri * ((1.0 / m_s) * (point - m_t)) * m_r);
	}

	/// applies just the rotation to the direction
	inline e3ga::vector transform_direction(const e3ga::vector &dir) const {
		return e3ga::_vector(m_r * dir * m_ri);
	}

	inline e3ga::vector transform_direction(const e3ga::__e3_ct__ &dir) const {
		return e3ga::_vector(m_r * dir * m_ri);
	}

	/// applies just the inverse rotation to the direction
	inline e3ga::vector inverse_transform_direction(const e3ga::vector &dir) const {
		return e3ga::_vector(m_ri * dir * m_r);
	}

	/// applies just the rotation to the normal
	inline e3ga::bivector transform_bivector(const e3ga::bivector &nl) const {
		return e3ga::_bivector(m_r * nl * m_ri);
	}

	/// applies just the inverse rotation to the normal
	inline e3ga::bivector inverse_transform_bivector(const e3ga::bivector &nl) const {
		return e3ga::_bivector(m_ri * nl * m_r);
	}

private:
	e3ga::vector m_t;
	GAIM_FLOAT m_s;

	e3ga::rotor m_r;
	e3ga::rotor m_ri;
};


#endif /* _LOCATOR_H_ */

