
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <math.h>
#include <string>
#include "locator.h"
#include "c3ga_util.h"

//using namespace e3ga;

locator::locator() {
}

locator::locator(const e3ga::vector &T, const GAIM_FLOAT S, const e3ga::rotor &R) {
	set(T, S, R);
}
/*: 
	m_t(T),  m_s(S) , m_r(R), m_ri(_rotor(inverse(R))) {
	e3ga::rotor Ri = _rotor(inverse(R));
	m_M.set(R * e3ga::e1 * Ri, R * e3ga::e2 * Ri, R * e3ga::e3 * Ri);
	m_Mi.set(Ri * e3ga::e1 * R, Ri * e3ga::e2 * R, Ri * e3ga::e3 * R);
}*/

locator::locator(const c3ga &TSR) {
	set_xf(TSR);
}


locator::locator(const locator &l) : m_t(l.m_t), m_s(l.m_s), m_M(l.m_M), m_Mi(l.m_Mi) {
}

locator::~locator() {
}


locator &locator::operator=(const locator &l) {
	if (this != &l) {
		m_t = l.m_t;
		m_s = l.m_s;
		m_M = l.m_M;
		m_Mi = l.m_Mi;
	}
	return *this;
}

void locator::set_xf(const c3ga &TSR) {
	e3ga::rotor R;
	conformal_TSR_versor_to_3DGA(TSR, m_t, m_s, R);
	e3ga::rotor Ri = _rotor(inverse(R));
	m_M.set(e3ga::_vector(R * e3ga::e1 * Ri), e3ga::_vector(R * e3ga::e2 * Ri), e3ga::_vector(R * e3ga::e3 * Ri));
	m_Mi.set(e3ga::_vector(Ri * e3ga::e1 * R), e3ga::_vector(Ri * e3ga::e2 * R), e3ga::_vector(Ri * e3ga::e3 * R));
}
					 
void locator::set(const e3ga::vector &T, const GAIM_FLOAT S, const e3ga::rotor &R) {
	m_t = T;
	m_s = S;
	e3ga::rotor Ri = _rotor(inverse(R));
	m_M.set(e3ga::_vector(R * e3ga::e1 * Ri), e3ga::_vector(R * e3ga::e2 * Ri), e3ga::_vector(R * e3ga::e3 * Ri));
	m_Mi.set(e3ga::_vector(Ri * e3ga::e1 * R), e3ga::_vector(Ri * e3ga::e2 * R), e3ga::_vector(Ri * e3ga::e3 * R));
}



