
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <math.h>
#include "e3la.h"

void s_quat2eulerXYZ(const GAIM_FLOAT q[4], GAIM_FLOAT xyz[3], const GAIM_FLOAT prev[3]) {
	GAIM_FLOAT A,B, c[4];
	GAIM_FLOAT p[4], t;

	A = -(GAIM_FLOAT)2.0 * (q[0] * q[1] + q[2] * q[3]);
	B = -q[0] * q[0] + q[1] * q[1] + q[2] * q[2] - q[3] * q[3];
	if ((A == 0) && (B == 0)) xyz[0] = 0.0;
	else {
		if (fabs(B) > fabs(A)) xyz[0] = (GAIM_FLOAT)atan(A / B);
		else xyz[0] = -(GAIM_FLOAT)atan(B / A) + (GAIM_FLOAT)(0.5 * M_PI);
	}
	if (prev && /* rotation continuity */
		((t = (GAIM_FLOAT)fabs(fmod((double)(xyz[0] - prev[0]), M_PI * 2.0))) > M_PI / 2.0) &&
		(t < M_PI * 2.0 -  M_PI / 2.0))
		xyz[0] += (GAIM_FLOAT)M_PI;

	c[0] = (GAIM_FLOAT)cos((GAIM_FLOAT)0.5 * xyz[0]);
	c[1] = (GAIM_FLOAT)sin((GAIM_FLOAT)0.5 * xyz[0]);
	if (c[1] < 0.0) {
		c[0] = -c[0];
		c[1] = -c[1];
	}
	p[0] = q[0] * c[0] + q[1] * c[1];
	p[2] = q[2] * c[0] - q[3] * c[1];
	p[3] = q[2] * c[1] + q[3] * c[0];

	t = (GAIM_FLOAT)sqrt(p[0] * p[0] + p[3] * p[3]);
	xyz[2] = (GAIM_FLOAT)2.0 * (GAIM_FLOAT)atan2(p[3] / t, p[0] / t);

	t = (GAIM_FLOAT)sqrt(p[0] * p[0] + p[2] * p[2]);
	xyz[1] = (GAIM_FLOAT)2.0 * (GAIM_FLOAT)atan2(p[2] / t, p[0] / t);
}

void s_quat2eulerYZX(const GAIM_FLOAT q[4], GAIM_FLOAT yzx[3], const GAIM_FLOAT prev[3]) {
	GAIM_FLOAT A, B, c[4];
	GAIM_FLOAT p[4], t;

	A = -(GAIM_FLOAT)2.0 * (q[0] * q[2] + q[1] * q[3]);
	B = -q[0] * q[0] - q[1] * q[1] + q[2] * q[2] + q[3] * q[3];
	if ((A == 0) && (B == 0)) yzx[0] = 0.0;
	else {
		if (fabs(B) > fabs(A)) yzx[0] = (GAIM_FLOAT)atan(A / B);
		else yzx[0] = -(GAIM_FLOAT)atan(B / A) + (GAIM_FLOAT)(0.5 * M_PI);
	}
	if (prev && /* rotation continuity */
		((t = (GAIM_FLOAT)fabs(fmod((double)(yzx[0] - prev[0]), M_PI * 2.0))) > M_PI / 2.0) &&
		(t < M_PI * 2.0 -  M_PI / 2.0))
		yzx[0] += (GAIM_FLOAT)M_PI;

	c[0] = (GAIM_FLOAT)cos((GAIM_FLOAT)0.5 * yzx[0]);
	c[2] = (GAIM_FLOAT)sin((GAIM_FLOAT)0.5 * yzx[0]);
	if (c[2] < 0.0) {
		c[0] = -c[0];
		c[2] = -c[2];
	}
	p[0] = q[0] * c[0] + q[2] * c[2];
	p[1] = q[1] * c[0] + q[3] * c[2];
	p[3] = -q[1] * c[2] + q[3] * c[0];

	t = (GAIM_FLOAT)sqrt(p[0] * p[0] + p[1] * p[1]);
	yzx[2] = (GAIM_FLOAT)2.0 * (GAIM_FLOAT)atan2(p[1] / t, p[0] / t);

	t = (GAIM_FLOAT)sqrt(p[0] * p[0] + p[3] * p[3]);
	yzx[1] = (GAIM_FLOAT)2.0 * (GAIM_FLOAT)atan2(p[3] / t, p[0] / t);
}

void s_quat2eulerYXZ(const GAIM_FLOAT q[4], GAIM_FLOAT yxz[3], const GAIM_FLOAT prev[3]) {
	GAIM_FLOAT A,B, c[4];
	GAIM_FLOAT p[4], t;

	A = -(GAIM_FLOAT)2.0 * (q[1] * q[3] - q[0] * q[2]);
	B = q[0] * q[0] - q[1] * q[1] - q[2] * q[2] + q[3] * q[3];
	if ((A == 0) && (B == 0)) yxz[0] = 0.0;
	else {
		if (fabs(B) > fabs(A)) yxz[0] = (GAIM_FLOAT)atan(A / B);
		else yxz[0] = -(GAIM_FLOAT)atan(B / A) + (GAIM_FLOAT)(0.5 * M_PI);
	}
	if (prev && /* rotation continuity */
		((t = (GAIM_FLOAT)fabs(fmod((double)(yxz[0] - prev[0]), M_PI * 2.0))) > M_PI / 2.0) &&
		(t < M_PI * 2.0 -  M_PI / 2.0))
		yxz[0] += (GAIM_FLOAT)M_PI;

	c[0] = (GAIM_FLOAT)cos((GAIM_FLOAT)0.5 * yxz[0]);
	c[2] = (GAIM_FLOAT)sin((GAIM_FLOAT)0.5 * yxz[0]);
	if (c[2] < 0.0) {
		c[0] = -c[0];
		c[2] = -c[2];
	}
	p[0] = q[0] * c[0] + q[2] * c[2];
	p[1] = q[1] * c[0] + q[3] * c[2];
	p[3] = -q[1] * c[2] + q[3] * c[0];

	t = (GAIM_FLOAT)sqrt(p[0] * p[0] + p[3] * p[3]);
	yxz[2] = (GAIM_FLOAT)2.0 * (GAIM_FLOAT)atan2(p[3] / t, p[0] / t);

	t = (GAIM_FLOAT)sqrt(p[0] * p[0] + p[1] * p[1]);
	yxz[1] = (GAIM_FLOAT)2.0 * (GAIM_FLOAT)atan2(p[1] / t, p[0] / t);
}

