
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef _LOCATOR_H_
#define _LOCATOR_H_

#include "c3ga.h"
#include "classes.h"

class locator {
public:
	locator();
	locator(const c3ga &TSR); ///< constructor from conformal TSR versor (uses conformal_TSR_versor_to_3DLA to convert)
	locator(const e3la_vec &T, const GAIM_FLOAT S, const e3la_mat &R);
	locator(const locator &l);
	virtual ~locator();

	locator &operator=(const locator &l);

	void set(const e3la_vec &T, const GAIM_FLOAT S, const e3la_mat &R); ///< sets transform
	void set_xf(const c3ga &TSR); ///< sets transform from conformal TSR versor (uses conformal_TSR_versor_to_3DLA to convert)

	inline const e3la_vec &get_translation() const {
		return m_t;
	}

	inline const e3la_mat &get_rotation() const {
		return m_r;
	}

	inline GAIM_FLOAT get_scale() const {
		return m_s;
	}

	inline e3la_vec transform_point(const e3la_vec &point) const {
		return m_s * (m_r * point) + m_t;
	}

	inline e3la_vec inverse_transform_point(const e3la_vec &point) const {
		return transposeMatrixVectorProduct(m_r, ((GAIM_FLOAT)1.0 / m_s) * (point - m_t));
	}

	/// applies just the rotation to the direction
	inline e3la_vec transform_direction(const e3la_vec &dir) const {
		return m_r * dir;
	}

	/// applies just the inverse rotation to the direction
	inline e3la_vec inverse_transform_direction(const e3la_vec &dir) const {
		return transposeMatrixVectorProduct(m_r, dir);
	}

	/// applies just the rotation to the normal
	inline e3la_vec transform_normal(const e3la_vec &nl) const {
		return m_r * nl;
	}

	/// applies just the inverse rotation to the normal
	inline e3la_vec inverse_transform_normal(const e3la_vec &nl) const {
		return transposeMatrixVectorProduct(m_r, nl);
	}

private:
	e3la_vec m_t;
	GAIM_FLOAT m_s;
	e3la_mat m_r;
};


#endif /* _LOCATOR_H_ */

