
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <math.h>

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#include <GL/gl.h>

#include "c3ga_util.h"
#include "p3ga.h"

using namespace p3ga;

void conformal_TSR_versor_to_P3GA(const c3ga &TSR, p3ga::om &M) {
	// extract rotation
	c3ga ie1 = TSR * (c3ga::e1 ^ c3ga::ni) / TSR; // ie1 = image of e1 ^ ni
	c3ga ie2 = TSR * (c3ga::e2 ^ c3ga::ni) / TSR; // ie2 = image of e2 ^ ni
	c3ga ie3 = TSR * (c3ga::e3 ^ c3ga::ni) / TSR; // ie3 = image of e3 ^ ni
	c3ga ie0 = TSR * (c3ga::no ^ c3ga::ni) / TSR; // ino = image of no ^ ni

	point image_of_e1(point_e1_e2_e3_e0, ie1[GRADE2][C3GA_E1_NI], ie1[GRADE2][C3GA_E2_NI], ie1[GRADE2][C3GA_E3_NI], ie1[GRADE2][C3GA_NO_NI]);
	point image_of_e2(point_e1_e2_e3_e0, ie2[GRADE2][C3GA_E1_NI], ie2[GRADE2][C3GA_E2_NI], ie2[GRADE2][C3GA_E3_NI], ie2[GRADE2][C3GA_NO_NI]);
	point image_of_e3(point_e1_e2_e3_e0, ie3[GRADE2][C3GA_E1_NI], ie3[GRADE2][C3GA_E2_NI], ie3[GRADE2][C3GA_E3_NI], ie3[GRADE2][C3GA_NO_NI]);
	point image_of_e0(point_e1_e2_e3_e0, ie0[GRADE2][C3GA_E1_NI], ie0[GRADE2][C3GA_E2_NI], ie0[GRADE2][C3GA_E3_NI], ie0[GRADE2][C3GA_NO_NI]);

	M.set(image_of_e1, image_of_e2, image_of_e3, image_of_e0);
}



