
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef _LOCATOR_H_
#define _LOCATOR_H_

#include "c3ga.h"
#include "p3la.h"

class locator {
public:
	locator();
	locator(const c3ga &TSR); ///< constructor from conformal TSR versor (uses conformal_TSR_versor_to_P3LA to convert)
	locator(const p3la_tm &tm, const p3la_tm &itm);
	locator(const locator &l);
	virtual ~locator();

	locator &operator=(const locator &l);


	void set_xf(const c3ga &TSR); ///< sets transform from conformal TSR versor (uses conformal_TSR_versor_to_3DLA to convert)

	inline GAIM_FLOAT get_scale() const {
		return m_tm.m44;
	}

	inline p3la_point get_translation() const {
		return m_tr;
	//	return p3la_point(m_tm.m14 / m_tm.m44, m_tm.m24 / m_tm.m44, m_tm.m34 / m_tm.m44, 1.0);
	}

	inline p3la_vec transform_direction(const p3la_vec &dir) const {
		return m_tm * dir;
	}
	inline p3la_vec inverse_transform_direction(const p3la_vec &dir) const {
		return m_itm * dir;
	}

	inline p3la_point transform_point(const p3la_point &dir) const {
		return m_tm * dir;
	}
	inline p3la_point inverse_transform_point(const p3la_point &dir) const {
		return m_itm * dir;
	}

	inline p3la_line transform_line(const p3la_line &dir) const {
		return m_tm * dir;
	}
	inline p3la_line inverse_transform_line(const p3la_line &dir) const {
		return m_itm * dir;
	}

	inline p3la_plane transform_plane(const p3la_plane &dir) const {
		return m_tm * dir;
	}
	inline p3la_plane inverse_transform_plane(const p3la_plane &dir) const {
		return m_itm * dir;
	}

private:
	p3la_tm m_tm, m_itm;
	p3la_point m_tr;
};


#endif /* _LOCATOR_H_ */

